#include <QStringList>
#include <QFileInfo>
#include <QUrl>

#include <QDebug>

#include "mimedb.h"
#include "playlistitem.h"

QMap<PlaylistItem::Type, QString> PlaylistItem::typeToStrMap(initStrings());

static PlaylistItem::Type guessType(QString uri)
{
    QUrl url(uri);

    if (url.isLocalFile())
    {
        QFileInfo info(url.path());
        if (info.isDir())
            return PlaylistItem::TypeDir;
    }

    if (url.scheme() == "rtsp")
        return PlaylistItem::TypeVideo;

    if ((url.scheme() == "http") || (url.scheme() == "https"))
        return PlaylistItem::TypeWebpage;

    QString mime = MimeDb::getForFileName(url.path());
    qDebug() << "mime:" << mime;
    if (mime.startsWith("video/"))
        return PlaylistItem::TypeVideo;
    if (mime.startsWith("audio/"))
        return PlaylistItem::TypeVideo;
    if (mime.startsWith("image/"))
        return PlaylistItem::TypeImage;
    if ((mime == "text/html") || (mime == "text/plain"))
        return PlaylistItem::TypeWebpage;
    if (mime == "application/pdf")
        return PlaylistItem::TypePdf;

    return PlaylistItem::TypeWebpage;
}

PlaylistItem::PlaylistItem(QString uri, PlaylistItem::Type type,
                           int time, bool href)
    : m_uri(uri)
    , m_type (type)
    , m_time(time)
    , m_href(href)
{
    if (m_type == PlaylistItem::TypeUndefined)
    {
        qDebug() << "Guessing Type";
        m_type = guessType(m_uri);
    }
}

QMap<PlaylistItem::Type, QString> PlaylistItem::initStrings()
{
    QMap<PlaylistItem::Type, QString> m;

    m[PlaylistItem::TypeRemote] = "Remote";
    m[PlaylistItem::TypeVideo] = "Video";
    m[PlaylistItem::TypeImage] = "Image";
    m[PlaylistItem::TypeWebpage] = "Webpage";
    m[PlaylistItem::TypePdf] = "PDF";
    m[PlaylistItem::TypeDocument] = "Document";
    m[PlaylistItem::TypeIdle] = "Idle";
    m[PlaylistItem::TypeNotSupported] = "Not Supported";

    return m;
}

QString PlaylistItem::typeToStr(PlaylistItem::Type type)
{
    return typeToStrMap[type];
}

PlaylistItem *PlaylistItem::idle()
{
    return new PlaylistItem(QString(), PlaylistItem::TypeIdle, 60);
}

PlaylistItem* PlaylistItem::fromXmlNode(QDomNode *node)
{
    QString uri;
    int time = 5;
    bool href = false;
    PlaylistItem::Type type = PlaylistItem::TypeUndefined;
    QDomNode tmp;

    QDomNamedNodeMap attributes = node->attributes();
    uri = attributes.namedItem("uri").nodeValue();

    tmp = attributes.namedItem("type");
    if (!tmp.isNull())
    {
        QString mediaType = tmp.nodeValue();
        if (mediaType == "url")
            type = PlaylistItem::TypeWebpage;
        else if (mediaType == "image")
            type = PlaylistItem::TypeImage;
        else if (mediaType == "audio")
            type = PlaylistItem::TypeVideo;
        else if (mediaType == "video")
            type = PlaylistItem::TypeVideo;
        else if (mediaType == "pdf")
            type = PlaylistItem::TypePdf;
        else if (mediaType == "document")
            type = PlaylistItem::TypeDocument;
    }

    tmp = attributes.namedItem("time");
    if (!tmp.isNull())
        time = tmp.nodeValue().toInt();
    else if (type == PlaylistItem::TypeVideo)
        time = 0;

    tmp = attributes.namedItem("href");
    if (!tmp.isNull())
        href = (tmp.nodeValue() == "1");


    return new PlaylistItem(uri, type, time, href);
}
