#include <unistd.h>
#include <sys/types.h>
#include <sys/utsname.h>

#include <qwebkitversion.h>
#include <QLocale>
#include <QNetworkCookie>
#include <QVariant>
#include <QSqlQuery>
#include <QDateTime>
#include <QWebElementCollection>
#include <QWebFrame>
#include <QNetworkReply>
#include <QMouseEvent>

#include <QDebug>

#include "webpage.h"
#include "config.h"
#include "settings.h"


SqliteCookieJar::SqliteCookieJar(QObject *parent)
    : QNetworkCookieJar(parent)
{
    QSqlQuery query("SELECT name,value,host,path,expiry,isSecure,isHttpOnly FROM moz_cookies");
    while (query.next())
    {
        QNetworkCookie c;
        c.setName(query.value(0).toByteArray());
        c.setValue(query.value(1).toByteArray());
        c.setDomain(query.value(2).toByteArray());
        c.setPath(query.value(3).toByteArray());
        c.setExpirationDate(QDateTime::fromTime_t(query.value(4).toInt()));
        c.setSecure(query.value(5).toBool());
        c.setHttpOnly(query.value(6).toBool());
        m_cookies.append(c);
    }

    setAllCookies(m_cookies);
}


QString CustomQWebPage::userAgentForUrl(const QUrl &url) const
{
    Settings *settings = Settings::load();

    QString user_agent = settings->get("main/user_agent").toString();
    if (! user_agent.isEmpty())
        return user_agent;

    QLocale l;
    struct utsname sysinfo;
    uname(&sysinfo);

    user_agent = QString("Mozilla/5.0 (X11; %1; %2) AppleWebKit/%3 (KHTML, like Gecko) "
                         "Chrome/" CHROME_VERSION " Safari/%3 LinutopKiosk/%4")
                    .arg(sysinfo.sysname, l.name(), qWebKitVersion(), VERSION);

    return user_agent;
}


Webpage::Webpage(PlaylistItem *item, QWidget *parent)
    : QWebView(parent)
    , Runnable(item->time())
{
    m_uri = QUrl(item->uri());
    loaded = false;

    setPage(&m_qwebpage);

    QObject::connect(this, SIGNAL(loadFinished(bool)),
                      static_cast<QWebView*>(this),
                      SLOT(onLoadFinished(bool)));
    installEventFilter(this);

    m_jar = new SqliteCookieJar(this);
    page()->networkAccessManager()->setCookieJar(m_jar);

    QObject::connect(
                page()->networkAccessManager(),
                SIGNAL(sslErrors(QNetworkReply*, const QList<QSslError> &)),
                this,
                SLOT(sslErrorHandler(QNetworkReply*, const QList<QSslError> &))
                );

    QWebView::setContextMenuPolicy(Qt::CustomContextMenu);
}


void Webpage::sslErrorHandler(QNetworkReply *reply,
                              const QList<QSslError> &error_list)
{ reply->ignoreSslErrors(); }


bool Webpage::eventFilter(QObject *obj, QEvent *event)
{
    setMouseTracking(true);
    switch (event->type())
    {
        case QEvent::MouseMove:
        case QEvent::MouseButtonPress:
            this->mousePressEvent(dynamic_cast<QMouseEvent*>(event));
        case QEvent::KeyPress:
            qobject_cast<QWidget *>(obj)->setCursor(Qt::BlankCursor);
            return true;
        default:
            return false;
    }
}


void Webpage::onLoadFinished(bool ok)
{
    loaded = ok;
    if (!ok) Q_EMIT end();
}


bool Webpage::startImpl(void)
{ load(m_uri); return true; }


bool Webpage::stopImpl(void)
{
    QWebElementCollection elements = page()->mainFrame()->findAllElements("embed");
    Q_FOREACH (QWebElement e, elements)
        e.removeFromDocument();
    return true;
}


bool Webpage::onTimeoutImpl()
{ Q_EMIT end(); return true; }


void Webpage::pauseImpl(void) {}


void Webpage::resumeImpl(void) {}


void Webpage::mousePressEvent(QMouseEvent* event)
{
    if (event->button() == Qt::RightButton)
        Q_EMIT rightclick();
    else
        QWebView::mousePressEvent(event);
}
